@@ -8,7 +8,7 @@
 #  data and digital camera RAW files.
 
 dlist=" g++  /usr/include/gtk-2.0/gtk/gtk.h  
-        /usr/include/FreeImage.h  /usr/lib*/libfreeimage*.so
+        /usr/include/tiffio.h  /usr/lib*/libtiff.so
         xdg-open  xdg-desktop-menu "
 
 error=0
@@ -29,7 +29,7 @@ echo "(package names depend on your flavor of Linux)."
 echo ""
 echo "g++                  the Gnu C++ compiler and linker"
 echo "libgtk2.0-dev        GTK graphics library (GUI base)"
-echo "libfreeimage-dev     FreeImage graphics library"
+echo "libtiff4-dev         TIFF file support library"
 echo "xdg-utils            LSB standard Linux utilities"
 echo ""
 
