@@ -1,7 +1,7 @@
 /**************************************************************************
    zfuncs.h      include file for zfuncs functions
 
-   Copyright 2006, 2007, 2008, 2009  Michael Cornelison
+   Copyright 2006, 2007, 2008, 2009, 2010  Michael Cornelison
    source URL:  kornelix.squarespace.com
    contact: kornelix@yahoo.de
    
@@ -20,7 +20,7 @@
 
 ***************************************************************************/
 
-//     zfuncs.h   version  v.2.28
+//     zfuncs.h   version  v.3.5.1
 
 #include <sys/time.h>
 #include <sys/stat.h>
@@ -69,8 +69,8 @@
 void appcrash(cchar * pMess, ... );                                        //  fatal exit - works like printf
 void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in xterm window
 void apppause();                                                           //  output message and wait for user
-void catch_signals();                                                      //  catch signals and do backtrace dump
 void apppause(cchar * pMess, ... );                                        //  same, works like printf
+void catch_signals();                                                      //  catch signals and do backtrace dump
 double get_seconds();                                                      //  seconds since 2000.01.01
 void start_timer(double &time0);                                           //  start a timer
 double get_timer(double &time0);                                           //  get elapsed time in seconds
@@ -83,17 +83,19 @@ int  zget_locked(int &param);                                              //  l
 void zput_locked(int &param, int value);                                   //  put and unlock multi-thread parameter
 int  zadd_locked(int &param, int incr);                                    //  increment multi-thread parameter
 void zsleep(double dsecs);                                                 //  sleep specified seconds
-char * zmalloc(size_t bytes, int log = 0);                                 //  malloc() with auto crash and log option
-void zfree(void *buff, int log = 0);                                       //  free() with log option
-char * command_output(int &contx, const char *command, ...);               //  get shell command output     v.2.3
+char * zmalloc(size_t bytes, cchar *tag = 0);                              //  malloc() with auto crash and log option
+void zfree(void *buff);                                                    //  free() with log option
+void zmalloc_log(int count);                                               //  turn zmalloc() log on/off       v.3.5
+void zmalloc_report();                                                     //  print total memory allocated    v.3.5
+char * command_output(int &contx, cchar *command, ...);                    //  get shell command output        v.2.3
 int command_status(int contx);                                             //  get exit status of command
 int signalProc(cchar * pname, cchar * signal);                             //  pause/resume/kill subprocess
-int runroot(const char *sucomm, const char *command);                      //  run command as root via su or sudo
-int checkinstall(const char *prog, ...);                                   //  check if programs are installed
+int runroot(cchar *sucomm, cchar *command);                                //  run command as root via su or sudo
+int checkinstall(cchar *prog, ...);                                        //  check if programs are installed
 char * fgets_trim(char * buff, int maxcc, FILE *, int bf = 0);             //  fgets + trim trailing \n \r (blanks)
 int samedirk(cchar *file1, cchar *file2);                                  //  returns 1 if files in same directory
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
-int zmondirk(const char *action, const char *dirk, char **file);           //  monitor a directory for changes   v.2.16
+int zmondirk(cchar *action, cchar *dirk, char **file);                     //  monitor a directory for changes   v.2.16
 
 //  measure CPU time spent in a function or code block within a function
 extern volatile double   cpu_profile_timer;                                //  internal data tables
@@ -135,16 +137,16 @@ void strToLower(char *dest, cchar *source);                                //  m
 void strToLower(char *string);                                             //  conv. string to lower case
 int  repl_1str(cchar *strin, char *strout, cchar *ssin, cchar *ssout);     //  copy string and replace 1 substring
 int  repl_Nstrs(cchar *strin, char *strout, ...);                          //  copy string and replace N substrings
-void strncpyx(char *out, const char *in, int ccin);                        //  conv. string to hex format
+void strncpyx(char *out, cchar *in, int ccin);                             //  conv. string to hex format
 void StripZeros(char *pNum);                                               //  1.230000E+12 --> 1.23E+12
 int  blank_null(cchar *string);                                            //  test for blank/null string
-char * strdupz(const char *string, int more = 0);                          //  duplicate string in nonvolatile memory
-int  strdupz(cchar *source, char *&dest, int more = 0);                    //  same + dest zmalloc if null or too short
+char * strdupz(cchar *string, int more = 0, cchar *tag = 0);               //  duplicate string in nonvolatile memory
+int  strdupz(cchar *source, char *&dest, int more = 0, cchar *tag = 0);    //  same + dest zmalloc if null or too short
 int  clean_escapes(char *string);                                          //  replace \x escapes with real characters
-int  utf8len(const char *utf8string);                                      //  get graphic cc for UTF8 string   v.2.3
-int  utf8substring(char *utf8out, const char *utf8in, int pos, int cc);    //  get graphic substring from UTF8 string
-int  utf8_check(const char *string);                                       //  check utf8 string for encoding errors
-int  utf8_position(const char *utf8in, int Nth);                           //  get byte position of Nth graphic char.
+int  utf8len(cchar *utf8string);                                           //  get graphic cc for UTF8 string   v.2.3
+int  utf8substring(char *utf8out, cchar *utf8in, int pos, int cc);         //  get graphic substring from UTF8 string
+int  utf8_check(cchar *string);                                            //  check utf8 string for encoding errors
+int  utf8_position(cchar *utf8in, int Nth);                                //  get byte position of Nth graphic char.
 
 //  bitmap functions ======================================================
 
@@ -174,7 +176,7 @@ int pvlist_find(pvlist *pv, cchar *entry);                                 //  f
 int pvlist_remove(pvlist *pv, cchar *entry);                               //  remove entry by name
 int pvlist_remove(pvlist *pv, int Nth);                                    //  remove entry by number (0...)
 int pvlist_count(pvlist *pv);                                              //  return entry count
-int pvlist_replace(pvlist *pv, int Nth, char *entry);                      //  replace Nth entry (0...)
+int pvlist_replace(pvlist *pv, int Nth, cchar *entry);                     //  replace Nth entry (0...)
 char * pvlist_get(pvlist *pv, int Nth);                                    //  return Nth entry (0...)
 int pvlist_sort(pvlist *pv);                                               //  sort list, ascending
 
@@ -210,15 +212,9 @@ int MemSort(char * RECS, int RL, int NR, int KEYS[][3], int NK);           //  m
 //  random number functions ===============================================
 
 int lrandz(int64 * seed);                                                  //  returns 0 to 0x7fffffff
+int lrandz();                                                              //  built-in seed
 double drandz(int64 * seed);                                               //  returns 0.0 to 0.99999...
-
-//  quick math functions ==================================================
-//  (2x-3x faster than C-lib, reduced accuracy, limited argument range)
-
-double qsine(double);                                                      //  quick sine
-double qcosine(double);                                                    //  quick cosine
-double qarcsine(double);                                                   //  quick arc sine
-double qsqrt(double);                                                      //  quick square root
+double drandz();                                                           //  built-in seed
 
 //  spline curve-fitting functions ========================================  v.2.11
 
@@ -232,17 +228,17 @@ double spline2(double x);                                                  //  r
 #define ZTXmaxent 1000                                                     //  max. translation strings
 #define ZTXmaxcc 2000                                                      //  max. cc per string
 
-int initz_appfiles(const char *appname, ...);                              //  find app dirs, copy initial files
-int ZTXinit(const char *lang);                                             //  setup for message translation
-const char *ZTX(const char *english);                                      //  get translation for English message
-const char * get_zuserdir();                                               //  get /home/user/.appname/
-const char * get_zdatadir();                                               //  get main install directory
-void showz_userguide();                                                    //  show help file in new process
-void showz_helpF1(const char *topic, ...);                                 //  show help topic(s) 
+int initz_appfiles(cchar *appname, ...);                                   //  find app dirs, copy initial files
+cchar * get_zuserdir();                                                    //  get /home/user/.appname/
+cchar * get_zdatadir();                                                    //  get main install directory
+void showz_userguide(cchar *context = 0);                                  //  show help file in new process
 void showz_readme();                                                       //  show README file in popup window
 void showz_changelog();                                                    //  show changelog file in popup window
 void showz_translations();                                                 //  show TRANSLATIONS file in popup window
-void zmake_launcher(const char *categories, const char *genericname);      //  create desktop icon/launcher   v.2.18
+void zmake_launcher(cchar *categories, cchar *genericname);                //  create desktop icon/launcher   v.2.18
+
+int ZTXinit(cchar *lang);                                                  //  setup for message translation
+cchar *ZTX(cchar *english);                                                //  get translation for English message
 
 /**************************************************************************
    GTK utility functions
@@ -258,7 +254,7 @@ void zlock();
 void zunlock();
 void zmainloop(int skip = 0);                                              //  do main loop, process menu events
 
-//  text edit/view window utilities
+//  text window print and read utilities
 void wprintx(GtkWidget *Win, int line, cchar *mess, cchar *font = 0);      //  write text to line, optional font
 void wprintf(GtkWidget *Win, int line, cchar *mess, ...);                  //  "printf" version
 void wprintf(GtkWidget *Win, cchar *mess, ... );                           //  "printf" to next line, scroll up
@@ -274,33 +270,26 @@ void wprintp(GtkWidget *Win);                                              //  p
 
 //  functions to simplify building menus, tool bars, status bars
 
-#define G_SIGNAL(window,event,func,arg)                 \
-  g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
+#define G_SIGNAL(window,event,func,arg) g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
 
-typedef void mtFunc(GtkWidget *, cchar *mname);                            //  menu or button response function
-
-GtkWidget * create_menubar(GtkWidget *vbox, int iconsize);                 //  create menubar in packing box
-GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname,                //  add menu item to menubar 
-                                       mtFunc func = 0);
-GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *label,               //  add submenu item to menu item
-                             cchar *icon = 0, mtFunc func = 0);            //  (optional icon and function)
-
-GtkWidget * create_toolbar(GtkWidget *vbox,                                //  horiz. toolbar in packing box (vert.)
-                           int iconsize = 24, int vert = 0);
-GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label,              //  add button with stock icon (gtk-quit)
-                               cchar *tip, cchar *icon, mtFunc func);      //    or custom icon (iconfile.png)
-
-GtkWidget * create_stbar(GtkWidget *vbox);                                 //  create status bar in packing box
-int stbar_message(GtkWidget *stbar, cchar *message);                       //  display message in status bar
+typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
+GtkWidget * create_menubar(GtkWidget *vbox);                                     //  create menubar in packing box
+GtkWidget * add_menubar_item(GtkWidget *mbar, cchar *mname, mtFunc func = 0);    //  add menu item to menubar 
+GtkWidget * add_submenu_item(GtkWidget *mitem, cchar *label, mtFunc func = 0);   //  add submenu item to menu item 
+GtkWidget * create_toolbar(GtkWidget *vbox, int iconsize = 24, int vert = 0);    //  horiz. toolbar in packing box (vert.)
+GtkWidget * add_toolbar_button(GtkWidget *tbar, cchar *label, cchar *tip,        //  add button with stock (gtk-quit) 
+                                 cchar *icon, mtFunc func);                      //    or custom icon (iconfile.png)
+GtkWidget * create_stbar(GtkWidget *vbox);                                       //  create status bar in packing box
+int stbar_message(GtkWidget *stbar, cchar *message);                             //  display message in status bar
 
 /**************************************************************************/
 
 //   functions to implement GTK dialogs with less complexity
-//   widget types: dialog, hbox, vbox, hsep, vsep, frame, scrwin, 
-//                 label, entry, edit, radio, check, button, togbutt, 
-//                 spin, combo, comboE, hscale, vscale, colorbutt
+//   widget types: dialog, hbox, vbox, hsep, vsep, frame, scrwin, label, entry, edit, radio, 
+//                 check, button, togbutt, spin, combo, comboE, hscale, vscale, colorbutt
 
 #define zdmaxwidgets 300
+#define zdmaxbutts 10
 #define zdsentinel 2138687954
 
 struct zwidget  {
@@ -319,10 +308,10 @@ struct zwidget  {
 struct zdialog  {
       int         sentinel;                        //  validity sintenel
       void        *eventCB;                        //  widget event user callback function
-      void        *complCB;                        //  dialog completion user callback function
       int         zstat;                           //  dialog status (from completion button)
       int         disabled;                        //  widget signals/events are disabled
-      zwidget     widget[zdmaxwidgets];            //  zdialog widgets (EOF = type = 0)
+      GtkWidget   *compbutt[zdmaxbutts];           //  dialog completion buttons
+      zwidget     widget[zdmaxwidgets];            //  dialog widgets (EOF = type = 0)
 };
 
 zdialog *zdialog_new(cchar *title, GtkWidget *parent, ...);                //  create a zdialog with opt. buttons
@@ -336,19 +325,20 @@ int zdialog_add_widget(zdialog *zd,                                        //  a
       cchar *type, cchar *name, cchar *pname,                              //  (alternative form)
       cchar *data, cchar *options);                                        //  "scc=nn|homog|expand|space=nn"
 
+GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                      //  GTK widget from zdialog widget name
+int zdialog_set_group(zdialog *zd, cchar *radio1, ...);                    //  set group ID for set of radio buttons
 int zdialog_resize(zdialog *zd, int width, int height);                    //  set size > widget sizes
 int zdialog_put_data(zdialog *zd, cchar *name, cchar *data);               //  put data in widget (entry, spin ...)
 cchar * zdialog_get_data(zdialog *zd, cchar *name);                        //  get widget data
-GtkWidget * zdialog_widget(zdialog *zd, cchar *name);                      //  GTK widget from zdialog widget name
 
 typedef int zdialog_event(zdialog *zd, cchar *name);                       //  widget event callback function
-typedef int zdialog_compl(zdialog *zd, int zstat);                         //  dialog completion callback function
-int zdialog_run(zdialog *zd, zdialog_event = 0, zdialog_compl = 0);        //  run dialog (compl func >> run parallel)
+int zdialog_run(zdialog *zd, zdialog_event = 0);                           //  run dialog, handle events
 
 int zdialog_send_event(zdialog *zd, cchar *event);                         //  send an event to an active dialog   v.2.17
 int zdialog_send_response(zdialog *zd, int zstat);                         //  complete a dialog, set status   v.2.23
 int zdialog_destroy(zdialog *zd);                                          //  destroy dialog (caller responsibility)
-int zdialog_free(zdialog *zd);                                             //  free zdialog memory ("")
+int zdialog_free(zdialog *zd);                                             //  free zdialog memory
+int zdialog_wait(zdialog *zd);                                             //  wait for dialog status or be destroyed
 int zdialog_goto(zdialog *zd, cchar *name);                                //  put cursor at named widget  v.2.23
 
 int zdialog_stuff(zdialog *zd, cchar *name, cchar *data);                  //  stuff string data into widget
@@ -358,6 +348,7 @@ int zdialog_stuff(zdialog *zd, cchar *name, double data);                  //  s
 int zdialog_fetch(zdialog *zd, cchar *name, char *data, int maxcc);        //  get string data from widget
 int zdialog_fetch(zdialog *zd, cchar *name, int  &data);                   //  get int data
 int zdialog_fetch(zdialog *zd, cchar *name, double &data);                 //  get double data
+int zdialog_fetch(zdialog *zd, cchar *name, float &data);                  //  get float data
 
 int zdialog_cb_app(zdialog *zd, cchar *name, cchar *data);                 //  append entry to combo drop down list
 int zdialog_cb_prep(zdialog *zd, cchar *name, cchar *data);                //  prepend entry to combo drop down list
@@ -369,18 +360,22 @@ int zdialog_cb_clear(zdialog *zd, cchar *name);                            //  c
 
 //   GTK misc. utility functioins
 
-int write_popup_text(cchar *action, cchar *text=0, int ww=0, int hh=0);    //  write text to popup window
-int popup_command(cchar *command, int ww=400, int hh=300);                 //  shell command output to popup window
-void zmessageACK(cchar *pMess, ... );                                      //  display message, wait for OK
-void zmessLogACK(cchar *pMess, ... );                                      //  same, with log to STDOUT
-int zmessageYN(cchar *pMess, ... );                                        //  display message, wait for YES or NO
-cchar * dialogText(cchar * title, cchar * inittext);                       //  get short text input from user
-char * zgetfile(cchar *title, cchar *file, cchar *rw, cchar *butt = 0);    //  file chooser dialog       v.2.3
-void showz_html(const char *url);                                          //  show html via preferred browser  v.2.18
+int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);  //  write text to popup window
+int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);                 //  shell command to popup window
 
-GtkWidget * zmessage_post(cchar *pMess, ...);                              //  display message indefinitely   v.2.6
+void zmessageACK(GtkWindow *, cchar *pMess, ... );                         //  display message, wait for OK
+void zmessLogACK(GtkWindow *, cchar *pMess, ... );                         //  same, with log to STDOUT
+int zmessageYN(GtkWindow *, cchar *pMess, ... );                           //  display message, wait for YES or NO
+GtkWidget * zmessage_post(GtkWindow *, cchar *pMess, ...);                 //  display message indefinitely   v.2.6
 void zmessage_kill(GtkWidget *dialog);                                     //  kill zmessage_post() message
 
+cchar * dialogText(GtkWindow *, cchar * title, cchar * inittext);          //  get short text input from user
+
+char * zgetfile(cchar *title, cchar *file, cchar *rw, cchar *butt = 0);    //  file chooser dialog       v.2.3
+char ** zgetfiles(cchar *title, cchar *file, cchar *butt = 0);             //  multiple file select dialog     v.2.29
+
+void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
+
 typedef void drag_drop_func(int x, int y, char *text);                     //  user function, get drag_drop text  v.2.19
 void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  connect window to user function
 
@@ -389,14 +384,15 @@ void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  c
 //   GDK/GTK image file utility functions
 //   for functions returning char *, caller responsible for zfree()
 
-GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double angle, int acolor = 0);  //  rotate pixbuf through any angle
+void print_imagefile(cchar *file, cchar *format = 0, cchar *orien = 0);    //  print image file (.jpg .png .tif)  v.3.4
+GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
 
 typedef void image_cbfunc(char *file);                                     //  callback function for clicked thumbnail
 char * image_gallery(cchar *filez, cchar *action, int Nth = 1,             //  display image gallery window, navigate,
-                                        image_cbfunc ufunc = 0);           //    do callback for clicked thumbnails
-void image_position(const char *file, int &posn, int &count);              //  get curr. image position, total image count
-int image_file_type(const char *file);                                     //  determine if directory or image file type
-char * image_thumbfile(const char *imagefile);                             //  get thumbnail filespec, create if missing
+                            image_cbfunc ufunc = 0);                       //    do callback for clicked thumbnails
+void image_position(cchar *file, int &posn, int &count);                   //  get curr. image position, total image count
+int image_file_type(cchar *file);                                          //  determine if directory or image file type
+char * image_thumbfile(cchar *imagefile);                                  //  get thumbnail filespec, create if missing
 GdkPixbuf * image_thumbnail(cchar *imagefile, int size = 0);               //  get thumbnail pixbuf, create if missing
 
 /**************************************************************************/
@@ -405,9 +401,9 @@ GdkPixbuf * image_thumbnail(cchar *imagefile, int size = 0);               //  g
 
 int initParmlist(int max);                                                 //  initz. parameter list
 int initz_userParms();                                                     //  load or initialize user parms
-int loadParms(const char * filename);                                      //  load parameters from a file
+int loadParms(cchar * filename);                                           //  load parameters from a file
 int loadParms();                                                           //   + user file select dialog
-int saveParms(const char * filename);                                      //  save parameters to a file
+int saveParms(cchar * filename);                                           //  save parameters to a file
 int saveParms();                                                           //   + user file select dialog
 int setParm(cchar * parmname, double parmval);                             //  set parameter value
 double getParm(cchar * parmname);                                          //  get parameter value
@@ -435,7 +431,7 @@ class xstring
       xstring(cchar * );                                                   //  string constructor
       xstring(const xstring &);                                            //  copy constructor
       ~xstring();                                                          //  destructor
-      operator cchar * () const { return xpp; }                            //  conversion operator (const char *)
+      operator cchar * () const { return xpp; }                            //  conversion operator (cchar *)
       xstring operator= (const xstring &);                                 //  operator =
       xstring operator= (cchar *);                                         //  operator =
       friend xstring operator+ (const xstring &, const xstring &);         //  operator +
@@ -534,7 +530,7 @@ class  Tree
    int         nsub;                                                       //  no. sub-nodes (Trees)
    Tree      **psub;                                                       //  pointer to sub-nodes
 public:
-   Tree(const char * name);                                                //  create Tree
+   Tree(cchar * name);                                                     //  create Tree
    ~Tree();                                                                //  destroy Tree
    int put(void * data, int cc, char * nodes[], int nn);                   //  put data by node names[]
    int put(void * data, int cc, int nodes[], int nn);                      //  put data by node numbers[]
